@@ -11,12 +11,16 @@
 #include <lua/bindings/inheritance.h>
 #include <lua/luaengine.h>
 
+#include <lua/bindings/async.h>
+
 #include <extensionsystem/iplugin.h>
 #include <extensionsystem/pluginmanager.h>
 
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectmanager.h>
 
 #include <utils/commandline.h>
+#include <utils/guardedcallback.h>
 #include <utils/layoutbuilder.h>
 
 #include <QJsonDocument>
@@ -26,6 +30,44 @@ using namespace Core;
 using namespace TextEditor;
 using namespace ProjectExplorer;
 
+namespace {
+
+class RequestWithResponse : public LanguageServerProtocol::JsonRpcMessage
+{
+    sol::main_function m_callback;
+    LanguageServerProtocol::MessageId m_id;
+
+public:
+    RequestWithResponse(const QJsonObject &obj, const sol::function &cb)
+        : LanguageServerProtocol::JsonRpcMessage(obj)
+        , m_callback(cb)
+    {
+        m_id = LanguageServerProtocol::MessageId(obj["id"]);
+    }
+
+    std::optional<LanguageServerProtocol::ResponseHandler> responseHandler() const override
+    {
+        if (!m_id.isValid()) {
+            qWarning() << "Invalid 'id' in request:" << toJsonObject();
+            return std::nullopt;
+        }
+
+        return LanguageServerProtocol::ResponseHandler{
+            m_id, [callback = m_callback](const JsonRpcMessage &msg) {
+                if (!callback.valid()) {
+                    qWarning() << "Invalid Lua callback";
+                    return;
+                }
+
+                auto result = ::Lua::void_safe_call(
+                    callback, ::Lua::toTable(callback.lua_state(), msg.toJsonObject()));
+                QTC_CHECK_EXPECTED(result);
+            }};
+    }
+};
+
+} // anonymous namespace
+
 namespace LanguageClient::Lua {
 
 static void registerLuaApi();
@@ -73,14 +115,16 @@ public:
         }
         m_process = new Process;
         m_process->setProcessMode(ProcessMode::Writer);
-        connect(m_process,
-                &Process::readyReadStandardError,
-                this,
-                &LuaLocalSocketClientInterface::readError);
-        connect(m_process,
-                &Process::readyReadStandardOutput,
-                this,
-                &LuaLocalSocketClientInterface::readOutput);
+        connect(
+            m_process,
+            &Process::readyReadStandardError,
+            this,
+            &LuaLocalSocketClientInterface::readError);
+        connect(
+            m_process,
+            &Process::readyReadStandardOutput,
+            this,
+            &LuaLocalSocketClientInterface::readOutput);
         connect(m_process, &Process::started, this, [this]() {
             this->LocalSocketClientInterface::startImpl();
             emit started();
@@ -136,8 +180,10 @@ class LuaClientWrapper;
 class LuaClientSettings : public BaseSettings
 {
     std::weak_ptr<LuaClientWrapper> m_wrapper;
+    QObject guard;
 
 public:
+    LuaClientSettings(const LuaClientSettings &wrapper);
     LuaClientSettings(const std::weak_ptr<LuaClientWrapper> &wrapper);
     ~LuaClientSettings() override = default;
 
@@ -159,24 +205,28 @@ enum class TransportType { StdIO, LocalSocket };
 
 class LuaClientWrapper : public QObject
 {
+    Q_OBJECT
 public:
     TransportType m_transportType{TransportType::StdIO};
     std::function<expected_str<void>(CommandLine &)> m_cmdLineCallback;
+    std::function<expected_str<void>(QString &)> m_initOptionsCallback;
+    sol::main_function m_asyncInitOptions;
+    bool m_isUpdatingAsyncOptions{false};
     AspectContainer *m_aspects{nullptr};
     QString m_name;
     Utils::Id m_settingsTypeId;
+    QString m_clientSettingsId;
     QString m_initializationOptions;
     CommandLine m_cmdLine;
     QString m_serverName;
     LanguageFilter m_languageFilter;
+    bool m_showInSettings;
     BaseSettings::StartBehavior m_startBehavior = BaseSettings::RequiresFile;
 
     std::optional<sol::protected_function> m_onInstanceStart;
     std::optional<sol::protected_function> m_startFailedCallback;
     QMap<QString, sol::protected_function> m_messageCallbacks;
 
-    LuaClientSettings *m_settings{nullptr};
-
 public:
     static BaseSettings::StartBehavior startBehaviorFromString(const QString &str)
     {
@@ -190,8 +240,6 @@ public:
         throw sol::error("Unknown start behavior: " + str.toStdString());
     }
 
-    void setSettings(LuaClientSettings *settings) { m_settings = settings; }
-
     LuaClientWrapper(const sol::table &options)
     {
         m_cmdLineCallback = addValue<CommandLine>(
@@ -204,6 +252,23 @@ public:
                 return cmdFromTable(res.get<sol::table>());
             });
 
+        m_initOptionsCallback = addValue<QString>(
+            options,
+            "initializationOptions",
+            m_initializationOptions,
+            [](const sol::protected_function_result &res) -> expected_str<QString> {
+                if (res.get_type(0) == sol::type::table)
+                    return ::Lua::toJsonString(res.get<sol::table>());
+                else if (res.get_type(0) == sol::type::string)
+                    return res.get<QString>(0);
+                return make_unexpected(QString("init callback did not return a table or string"));
+            });
+
+        if (auto initOptionsTable = options.get<sol::optional<sol::table>>("initializationOptions"))
+            m_initializationOptions = ::Lua::toJsonString(*initOptionsTable);
+        else if (auto initOptionsString = options.get<sol::optional<QString>>("initializationOptions"))
+            m_initializationOptions = *initOptionsString;
+
         m_name = options.get<QString>("name");
         m_settingsTypeId = Utils::Id::fromString(QString("Lua_%1").arg(m_name));
         m_serverName = options.get_or<QString>("serverName", "");
@@ -235,21 +300,7 @@ public:
                     m_languageFilter.mimeTypes.push_back(v.as<QString>());
         }
 
-        auto initOptionsTable = options.get<sol::optional<sol::table>>("initializationOptions");
-        if (initOptionsTable) {
-            QJsonValue json = ::Lua::LuaEngine::toJson(*initOptionsTable);
-            QJsonDocument doc;
-            if (json.isArray()) {
-                doc.setArray(json.toArray());
-                m_initializationOptions = QString::fromUtf8(doc.toJson());
-            } else if (json.isObject()) {
-                doc.setObject(json.toObject());
-                m_initializationOptions = QString::fromUtf8(doc.toJson());
-            }
-        }
-        auto initOptionsString = options.get<sol::optional<QString>>("initializationOptions");
-        if (initOptionsString)
-            m_initializationOptions = *initOptionsString;
+        m_showInSettings = options.get<std::optional<bool>>("showInSettings").value_or(true);
 
         // get<sol::optional<>> because on MSVC, get_or(..., nullptr) fails to compile
         m_aspects = options.get<sol::optional<AspectContainer *>>("settings").value_or(nullptr);
@@ -268,7 +319,7 @@ public:
             [this](Client *c) {
                 auto luaClient = qobject_cast<LuaClient *>(c);
                 if (luaClient && luaClient->m_settingsId == m_settingsTypeId && m_onInstanceStart) {
-                    QTC_CHECK(::Lua::LuaEngine::void_safe_call(*m_onInstanceStart, c));
+                    QTC_CHECK(::Lua::void_safe_call(*m_onInstanceStart, c));
                     updateMessageCallbacks();
                 }
             });
@@ -278,6 +329,13 @@ public:
             &LanguageClientManager::clientRemoved,
             this,
             &LuaClientWrapper::onClientRemoved);
+
+        if (auto asyncInit = options.get<sol::optional<sol::main_function>>(
+                "initializationOptionsAsync")) {
+            m_asyncInitOptions = *asyncInit;
+            QMetaObject::invokeMethod(
+                this, &LuaClientWrapper::updateAsyncOptions, Qt::QueuedConnection);
+        }
     }
 
     void onClientRemoved(Client *c, bool unexpected)
@@ -287,7 +345,7 @@ public:
             return;
 
         if (unexpected && m_startFailedCallback) {
-            QTC_CHECK_EXPECTED(::Lua::LuaEngine::void_safe_call(*m_startFailedCallback));
+            QTC_CHECK_EXPECTED(::Lua::void_safe_call(*m_startFailedCallback));
         }
     }
 
@@ -317,11 +375,11 @@ public:
     Layouting::LayoutModifier settingsLayout()
     {
         if (m_aspects)
-            return [this](Layouting::Layout *iface) { m_aspects->addToLayout(*iface); };
+            return [this](Layouting::Layout *iface) { m_aspects->addToLayoutImpl(*iface); };
         return {};
     }
 
-    void registerMessageCallback(const QString &msg, const sol::function &callback)
+    void registerMessageCallback(const QString &msg, const sol::main_function &callback)
     {
         m_messageCallbacks.insert(msg, callback);
         updateMessageCallbacks();
@@ -329,7 +387,7 @@ public:
 
     void updateMessageCallbacks()
     {
-        for (Client *c : LanguageClientManager::clientsForSetting(m_settings)) {
+        for (Client *c : LanguageClientManager::clientsForSettingId(m_clientSettingsId)) {
             if (!c)
                 continue;
             for (const auto &[msg, func] : m_messageCallbacks.asKeyValueRange()) {
@@ -341,7 +399,7 @@ public:
                             return;
 
                         auto func = self->m_messageCallbacks.value(name);
-                        auto table = ::Lua::LuaEngine::toTable(func.lua_state(), m.toJsonObject());
+                        auto table = ::Lua::toTable(func.lua_state(), m.toJsonObject());
                         auto result = func.call(table);
                         if (!result.valid()) {
                             qWarning() << "Error calling message callback for:" << name << ":"
@@ -354,16 +412,90 @@ public:
 
     void sendMessage(const sol::table &message)
     {
-        const QJsonValue messageValue = ::Lua::LuaEngine::toJson(message);
+        const QJsonValue messageValue = ::Lua::toJson(message);
         if (!messageValue.isObject())
             throw sol::error("Message is not an object");
-        const LanguageServerProtocol::JsonRpcMessage jsonrpcmessage(messageValue.toObject());
-        for (Client *c : LanguageClientManager::clientsForSetting(m_settings)) {
+
+        const LanguageServerProtocol::JsonRpcMessage request(messageValue.toObject());
+        for (Client *c : LanguageClientManager::clientsForSettingId(m_clientSettingsId)) {
             if (c)
-                c->sendMessage(jsonrpcmessage);
+                c->sendMessage(request);
         }
     }
 
+    QList<Client *> clientsForDocument(Core::IDocument *document)
+    {
+        QList<Client *> result;
+        if (m_startBehavior == BaseSettings::RequiresProject) {
+            Project *project = ProjectManager::projectForFile(document->filePath());
+            const auto clients = LanguageClientManager::clientsForSettingId(m_clientSettingsId);
+            result = Utils::filtered(clients, [project](Client *c) {
+                return c && c->project() == project;
+            });
+        }
+        else
+            result = LanguageClientManager::clientsForSettingId(m_clientSettingsId);
+
+        return Utils::filtered(result, [](Client *c) { return c->reachable(); });
+    }
+
+    void sendMessageForDocument(Core::IDocument *document, const sol::table &message)
+    {
+        const QJsonValue messageValue = ::Lua::toJson(message);
+        if (!messageValue.isObject())
+            throw sol::error("Message is not an object");
+
+        const LanguageServerProtocol::JsonRpcMessage request(messageValue.toObject());
+
+        auto clients = clientsForDocument(document);
+        QTC_CHECK(clients.size() == 1);
+
+        for (Client *c : clients) {
+            if (c)
+                c->sendMessage(request);
+        }
+    }
+
+    void sendMessageWithIdForDocument_cb(
+        TextEditor::TextDocument *document, const sol::table &message, const sol::main_function callback)
+    {
+        const QJsonValue messageValue = ::Lua::toJson(message);
+        if (!messageValue.isObject())
+            throw sol::error("Message is not an object");
+
+        QJsonObject obj = messageValue.toObject();
+        obj["id"] = QUuid::createUuid().toString();
+
+        const RequestWithResponse request{obj, callback};
+
+        auto clients = clientsForDocument(document);
+
+        QTC_ASSERT(clients.size() != 0, throw sol::error("No client for document found"));
+        QTC_ASSERT(clients.size() == 1, throw sol::error("Multiple clients for document found"));
+        QTC_ASSERT(clients.front(), throw sol::error("Client is null"));
+
+        clients.front()->sendMessage(request);
+    }
+
+    void updateAsyncOptions()
+    {
+        if (m_isUpdatingAsyncOptions)
+            return;
+        QTC_ASSERT(m_asyncInitOptions, return);
+        m_isUpdatingAsyncOptions = true;
+        std::function<void(sol::object)> cb = guardedCallback(this, [this](sol::object options) {
+            if (options.is<sol::table>())
+                m_initializationOptions = ::Lua::toJsonString(options.as<sol::table>());
+            else if (options.is<QString>())
+                m_initializationOptions = options.as<QString>();
+
+            emit optionsChanged();
+            m_isUpdatingAsyncOptions = false;
+        });
+
+        ::Lua::Async::start<sol::object>(m_asyncInitOptions, cb);
+    }
+
     void updateOptions()
     {
         if (m_cmdLineCallback) {
@@ -371,6 +503,18 @@ public:
             if (!result)
                 qWarning() << "Error applying option callback:" << result.error();
         }
+        if (m_initOptionsCallback) {
+            expected_str<void> result = m_initOptionsCallback(m_initializationOptions);
+            if (!result)
+                qWarning() << "Error applying init option callback:" << result.error();
+
+            // Right now there is only one option that needs to be mirrored to the LSP Settings,
+            // so we only emit optionsChanged() here. If another setting should need to be dynamic
+            // optionsChanged() needs to be called for it as well, but only once per updateOptions()
+            emit optionsChanged();
+        }
+        if (m_asyncInitOptions)
+            updateAsyncOptions();
     }
 
     static CommandLine cmdFromTable(const sol::table &tbl)
@@ -438,8 +582,23 @@ public:
         }
         return nullptr;
     }
+
+signals:
+    void optionsChanged();
 };
 
+LuaClientSettings::LuaClientSettings(const LuaClientSettings &other)
+    : BaseSettings::BaseSettings(other)
+    , m_wrapper(other.m_wrapper)
+{
+    if (auto w = m_wrapper.lock()) {
+        QObject::connect(w.get(), &LuaClientWrapper::optionsChanged, &guard, [this] {
+            if (auto w = m_wrapper.lock())
+                m_initializationOptions = w->m_initializationOptions;
+        });
+    }
+}
+
 LuaClientSettings::LuaClientSettings(const std::weak_ptr<LuaClientWrapper> &wrapper)
     : m_wrapper(wrapper)
 {
@@ -449,6 +608,11 @@ LuaClientSettings::LuaClientSettings(const std::weak_ptr<LuaClientWrapper> &wrap
         m_languageFilter = w->m_languageFilter;
         m_initializationOptions = w->m_initializationOptions;
         m_startBehavior = w->m_startBehavior;
+        m_showInSettings = w->m_showInSettings;
+        QObject::connect(w.get(), &LuaClientWrapper::optionsChanged, &guard, [this] {
+            if (auto w = m_wrapper.lock())
+                m_initializationOptions = w->m_initializationOptions;
+        });
     }
 }
 
@@ -456,8 +620,14 @@ bool LuaClientSettings::applyFromSettingsWidget(QWidget *widget)
 {
     BaseSettings::applyFromSettingsWidget(widget);
 
-    if (auto w = m_wrapper.lock())
+    if (auto w = m_wrapper.lock()) {
+        w->m_name = m_name;
+        if (!w->m_initOptionsCallback)
+            w->m_initializationOptions = m_initializationOptions;
+        w->m_languageFilter = m_languageFilter;
+        w->m_startBehavior = m_startBehavior;
         w->applySettings();
+    }
 
     return true;
 }
@@ -475,7 +645,8 @@ void LuaClientSettings::fromMap(const Utils::Store &map)
     BaseSettings::fromMap(map);
     if (auto w = m_wrapper.lock()) {
         w->m_name = m_name;
-        w->m_initializationOptions = m_initializationOptions;
+        if (!w->m_initOptionsCallback)
+            w->m_initializationOptions = m_initializationOptions;
         w->m_languageFilter = m_languageFilter;
         w->m_startBehavior = m_startBehavior;
         w->fromMap(map);
@@ -508,7 +679,10 @@ BaseClientInterface *LuaClientSettings::createInterface(ProjectExplorer::Project
 
 static void registerLuaApi()
 {
-    ::Lua::LuaEngine::registerProvider("LSP", [](sol::state_view lua) -> sol::object {
+    ::Lua::registerProvider("LSP", [](sol::state_view lua) -> sol::object {
+        sol::table async = lua.script("return require('async')", "_process_").get<sol::table>();
+        sol::function wrap = async["wrap"];
+
         sol::table result = lua.create_table();
 
         auto wrapperClass = result.new_usertype<LuaClientWrapper>(
@@ -520,38 +694,67 @@ static void registerLuaApi()
                         return sol::lua_nil;
                     return c->m_onInstanceStart.value();
                 },
-                [](LuaClientWrapper *c, const sol::function &f) { c->m_onInstanceStart = f; }),
+                [](LuaClientWrapper *c, const sol::main_function &f) { c->m_onInstanceStart = f; }),
             "registerMessage",
             &LuaClientWrapper::registerMessageCallback,
             "sendMessage",
             &LuaClientWrapper::sendMessage,
+            "sendMessageForDocument",
+            &LuaClientWrapper::sendMessageForDocument,
+            "sendMessageWithIdForDocument_cb",
+            &LuaClientWrapper::sendMessageWithIdForDocument_cb,
             "create",
-            [](const sol::table &options) -> std::shared_ptr<LuaClientWrapper> {
-                auto luaClient = std::make_shared<LuaClientWrapper>(options);
-                auto client = new LuaClientSettings(luaClient);
-                luaClient->setSettings(client);
+            [](const sol::main_table &options) -> std::shared_ptr<LuaClientWrapper> {
+                auto luaClientWrapper = std::make_shared<LuaClientWrapper>(options);
+                auto clientSettings = new LuaClientSettings(luaClientWrapper);
 
                 // The order is important!
                 // First restore the settings ...
                 const QList<Utils::Store> savedSettings
-                    = LanguageClientSettings::storesBySettingsType(luaClient->m_settingsTypeId);
+                    = LanguageClientSettings::storesBySettingsType(
+                        luaClientWrapper->m_settingsTypeId);
 
                 if (!savedSettings.isEmpty())
-                    client->fromMap(savedSettings.first());
+                    clientSettings->fromMap(savedSettings.first());
 
                 // ... then register the settings.
-                LanguageClientManager::registerClientSettings(client);
+                LanguageClientManager::registerClientSettings(clientSettings);
+                luaClientWrapper->m_clientSettingsId = clientSettings->m_id;
 
                 // and the client type.
                 ClientType type;
-                type.id = client->m_settingsTypeId;
-                type.name = luaClient->m_name;
+                type.id = clientSettings->m_settingsTypeId;
+                type.name = luaClientWrapper->m_name;
                 type.userAddable = false;
                 LanguageClientSettings::registerClientType(type);
 
-                return luaClient;
+                return luaClientWrapper;
+            },
+            "documentVersion",
+            [](LuaClientWrapper *wrapper,
+               const Utils::FilePath &path) -> std::tuple<bool, std::variant<int, QString>> {
+                auto clients = wrapper->clientsForDocument(
+                    TextEditor::TextDocument::textDocumentForFilePath(path));
+                if (clients.empty())
+                    return {false, "No client found."};
+
+                return {true, clients.first()->documentVersion(path)};
+            },
+
+            "hostPathToServerUri",
+            [](LuaClientWrapper *wrapper, const Utils::FilePath &path) -> std::tuple<bool, QString> {
+                auto clients = wrapper->clientsForDocument(
+                    TextEditor::TextDocument::textDocumentForFilePath(path));
+                if (clients.empty())
+                    return {false, "No client found."};
+
+                return {true, clients.first()->hostPathToServerUri(path).toString()};
             });
 
+        wrapperClass["sendMessageWithIdForDocument"]
+            = wrap(wrapperClass["sendMessageWithIdForDocument_cb"].get<sol::function>())
+                  .get<sol::function>();
+
         return result;
     });
 }
