@@ -17,7 +17,8 @@
 #include <projectexplorer/projectmanager.h>
 #include <projectexplorer/projecttree.h>
 
-#include <utils/process.h>
+#include <utils/fileutils.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
 
 #include <QAction>
@@ -27,9 +28,9 @@
 #include <QMessageBox>
 #include <QScopedPointer>
 #include <QSharedData>
-#include <QSharedPointer>
 #include <QProcessEnvironment>
 #include <QTextCodec>
+#include <QTimer>
 
 using namespace Core;
 using namespace Utils;
@@ -308,8 +309,10 @@ void StateListener::slotStateChanged()
         state.clearPatchFile(); // Need a repository to patch
 
     qCDebug(stateLog).noquote() << "VC:" << (vc ? vc->displayName() : QString("None")) << state;
-    EditorManager::updateWindowTitles();
-    emit stateChanged(state, vc);
+    QTimer::singleShot(500, this, [this, state, vc] {
+        EditorManager::updateWindowTitles();
+        emit stateChanged(state, vc);
+    });
 }
 
 } // namespace Internal
@@ -329,8 +332,8 @@ public:
     Qt Creator's state relevant to VCS plugins is a tuple of
 
     \list
-    \li Current file and it's version system control/top level
-    \li Current project and it's version system control/top level
+    \li Current file and its version control system top level
+    \li Current project and its version control system top level
     \endlist
 
     \sa VcsBase::VcsBasePlugin
@@ -375,7 +378,7 @@ FilePath VcsBasePluginState::currentFileDirectory() const
 QString VcsBasePluginState::relativeCurrentFile() const
 {
     QTC_ASSERT(hasFile(), return {});
-    return data->m_state.currentFile.relativeChildPath(data->m_state.currentFileTopLevel).toString();
+    return data->m_state.currentFile.relativeChildPath(data->m_state.currentFileTopLevel).path();
 }
 
 QString VcsBasePluginState::currentPatchFile() const
@@ -489,35 +492,33 @@ VCSBASE_EXPORT QDebug operator<<(QDebug in, const VcsBasePluginState &state)
 
     When triggering an action, a copy of the state should be made to
     keep it, as it may rapidly change due to context changes, etc.
-
-    The class also detects the VCS plugin submit editor closing and calls
-    the virtual submitEditorAboutToClose() to trigger the submit process.
 */
 
-bool VcsBasePluginPrivate::supportsRepositoryCreation() const
+bool VersionControlBase::supportsRepositoryCreation() const
 {
     return supportsOperation(IVersionControl::CreateRepositoryOperation);
 }
 
 static Internal::StateListener *m_listener = nullptr;
 
-VcsBasePluginPrivate::VcsBasePluginPrivate(const Context &context)
+VersionControlBase::VersionControlBase(const Context &context)
     : m_context(context)
 {
-    Internal::VcsPlugin *plugin = Internal::VcsPlugin::instance();
-    connect(plugin, &Internal::VcsPlugin::submitEditorAboutToClose,
-            this, [this](VcsBaseSubmitEditor *submitEditor, bool *result) {
-        if (submitEditor == m_submitEditor) {
-            *result = submitEditor->promptSubmit(this);
-            if (*result)
+    EditorManager::addCloseEditorListener([this](IEditor *editor) {
+        bool result = true;
+        if (editor == m_submitEditor) {
+            result = m_submitEditor->promptSubmit(this);
+            if (result)
                 discardCommit();
         }
+        return result;
     });
+
     // First time: create new listener
     if (!m_listener)
-        m_listener = new Internal::StateListener(plugin);
+        m_listener = new Internal::StateListener(Internal::VcsPlugin::instance());
     connect(m_listener, &Internal::StateListener::stateChanged,
-            this, &VcsBasePluginPrivate::slotStateChanged);
+            this, &VersionControlBase::slotStateChanged);
     // VCSes might have become (un-)available, so clear the VCS directory cache
     connect(this, &IVersionControl::configurationChanged,
             VcsManager::instance(), &VcsManager::clearVersionControlCache);
@@ -525,13 +526,13 @@ VcsBasePluginPrivate::VcsBasePluginPrivate(const Context &context)
             m_listener, &Internal::StateListener::slotStateChanged);
 }
 
-void VcsBasePluginPrivate::extensionsInitialized()
+void VersionControlBase::extensionsInitialized()
 {
     // Initialize enable menus.
     m_listener->slotStateChanged();
 }
 
-void VcsBasePluginPrivate::slotStateChanged(const Internal::State &newInternalState, Core::IVersionControl *vc)
+void VersionControlBase::slotStateChanged(const Internal::State &newInternalState, IVersionControl *vc)
 {
     if (vc == this) {
         // We are directly affected: Change state
@@ -553,13 +554,13 @@ void VcsBasePluginPrivate::slotStateChanged(const Internal::State &newInternalSt
     }
 }
 
-const VcsBasePluginState &VcsBasePluginPrivate::currentState() const
+const VcsBasePluginState &VersionControlBase::currentState() const
 {
     return m_state;
 }
 
-VcsCommand *VcsBasePluginPrivate::createInitialCheckoutCommand(const QString &url,
-                                                               const Utils::FilePath &baseDirectory,
+VcsCommand *VersionControlBase::createInitialCheckoutCommand(const QString &url,
+                                                               const FilePath &baseDirectory,
                                                                const QString &localName,
                                                                const QStringList &extraArgs)
 {
@@ -570,7 +571,7 @@ VcsCommand *VcsBasePluginPrivate::createInitialCheckoutCommand(const QString &ur
     return nullptr;
 }
 
-bool VcsBasePluginPrivate::enableMenuAction(ActionState as, QAction *menuAction) const
+bool VersionControlBase::enableMenuAction(ActionState as, QAction *menuAction) const
 {
     qCDebug(baseLog) << "enableMenuAction" << menuAction->text() << as;
     switch (as) {
@@ -591,42 +592,42 @@ bool VcsBasePluginPrivate::enableMenuAction(ActionState as, QAction *menuAction)
     return true;
 }
 
-QString VcsBasePluginPrivate::commitDisplayName() const
+QString VersionControlBase::commitDisplayName() const
 {
     //: Name of the "commit" action of the VCS
     return Tr::tr("Commit", "name of \"commit\" action of the VCS.");
 }
 
-QString VcsBasePluginPrivate::commitAbortTitle() const
+QString VersionControlBase::commitAbortTitle() const
 {
     return Tr::tr("Close Commit Editor");
 }
 
-QString VcsBasePluginPrivate::commitAbortMessage() const
+QString VersionControlBase::commitAbortMessage() const
 {
     return Tr::tr("Closing this editor will abort the commit.");
 }
 
-QString VcsBasePluginPrivate::commitErrorMessage(const QString &error) const
+QString VersionControlBase::commitErrorMessage(const QString &error) const
 {
     if (error.isEmpty())
         return Tr::tr("Cannot commit.");
     return Tr::tr("Cannot commit: %1.").arg(error);
 }
 
-void VcsBasePluginPrivate::commitFromEditor()
+void VersionControlBase::commitFromEditor()
 {
     QTC_ASSERT(m_submitEditor, return);
     m_submitEditor->accept(this);
 }
 
-bool VcsBasePluginPrivate::promptBeforeCommit()
+bool VersionControlBase::promptBeforeCommit()
 {
     return DocumentManager::saveAllModifiedDocuments(Tr::tr("Save before %1?")
                                                      .arg(commitDisplayName().toLower()));
 }
 
-void VcsBasePluginPrivate::promptToDeleteCurrentFile()
+void VersionControlBase::promptToDeleteCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
@@ -645,7 +646,7 @@ static inline bool ask(QWidget *parent, const QString &title, const QString &que
     return QMessageBox::question(parent, title, question, QMessageBox::Yes|QMessageBox::No, defaultButton) == QMessageBox::Yes;
 }
 
-void VcsBasePluginPrivate::createRepository()
+void VersionControlBase::createRepository()
 {
     QTC_ASSERT(supportsOperation(IVersionControl::CreateRepositoryOperation), return);
     // Find current starting directory
@@ -682,17 +683,17 @@ void VcsBasePluginPrivate::createRepository()
     }
 }
 
-void VcsBasePluginPrivate::setSubmitEditor(VcsBaseSubmitEditor *submitEditor)
+void VersionControlBase::setSubmitEditor(VcsBaseSubmitEditor *submitEditor)
 {
     m_submitEditor = submitEditor;
 }
 
-VcsBaseSubmitEditor *VcsBasePluginPrivate::submitEditor() const
+VcsBaseSubmitEditor *VersionControlBase::submitEditor() const
 {
     return m_submitEditor;
 }
 
-bool VcsBasePluginPrivate::raiseSubmitEditor() const
+bool VersionControlBase::raiseSubmitEditor() const
 {
     if (!m_submitEditor)
         return false;
@@ -700,7 +701,7 @@ bool VcsBasePluginPrivate::raiseSubmitEditor() const
     return true;
 }
 
-void VcsBasePluginPrivate::discardCommit()
+void VersionControlBase::discardCommit()
 {
 }
 
